#### FROM HORACE ###
#
# NeoScript [TM] development source code
#
# Tcl file that, when debugging is true, gets sourced every time a webpage 
# is pulled up that contains Neoscript server-side code. 
#
# If debugging is false, the file is loaded in only once by the
# "mother thread".
#
# $Id: neotrack.tcl,v 1.1 1996/06/20 07:35:33 kunkee Exp $
#

proc neotrack {{timeout 30} {thisdb NeoTrack}} {
    global webenv
    if {$timeout ==  "-"} {
        set timeout 30
    }
    set ctime [clock seconds]
    set to $webenv(DOCUMENT_URI)
    if {[info exists webenv(HTTP_REFERER)]} {
        set from $webenv(HTTP_REFERER)
    } else {
        set from "-"
    }
    set visitor_ip $webenv(REMOTE_ADDR)
    if {[dbexists $thisdb]} {
        set keylist [lsort -ascii -decreasing [dbkeys $thisdb *_${visitor_ip}]] 
        if {[llength $keylist] >= 0} {
            set key [lindex $keylist 0]
            dbfetch $thisdb $key record
            if {[llength [array names record]] > 0} {
                set fieldlist [lsort -integer -decreasing [array names record]]
                set lastvisit [lindex $fieldlist 0]
                set last_timeout [lindex $record($lastvisit) 2]
                if {[expr $ctime - $lastvisit] <= [expr 60 * $last_timeout]} {
                    #
                    # NOT a NEW visitor
                    #
                    set record($ctime) [list $from $to $timeout]
                    dbstore $thisdb $key record
                    return 0
                }
            }
        }
    }
    #
    # IS a NEW visitor
    #
    set key ${ctime}_${visitor_ip}
    set newrecord($ctime) [list $from $to $timeout]
    dbstore $thisdb $key newrecord
    #
    # now make the pageEntry record addition or update
    #
    set key [clock format $ctime -format "%Y%m%d"]$to
    set from [lindex [split $from ?] 0]
    if {[dbexists NeoTrackEntry]} {
        if {[dbfetch NeoTrackEntry $key parray]} {
            incr parray(daycount)
            if {[info exists parray($from)]} {
                incr parray($from)
            } else {
                set parray($from) 1
            }
        } else {
            set parray(daycount) 1
            set parray($from) 1
        }
    } else {
        set parray(daycount) 1
        set parray($from) 1
    }
    dbstore NeoTrackEntry $key parray
    return 1
}


proc neotrack_report {key {thisdb NeoTrack}} {
    dbfetch $thisdb $key record
    set key_components [split $key _]
    set ctime [lindex $key_components 0]
    set visitor_ip [lindex $key_components 1]

    set page_list [lsort -integer -increasing [array names record]]
    set page_list_len [llength $page_list]
    set page_index 0

    html "<b>$visitor_ip</b> visited on <b>[clock format $ctime]</b><br>"
    html "<table border=\"1\" cellpadding=\"5\">"
    html "<tr><td valign=\"top\"><b>From</b></td>"
    html "<td valign=\"top\"><b>Page URI</b></td>"
    html "<td valign=\"top\"><b>Enrty Time</b></td>"
    html "<td valign=\"top\"><b>Visit Length</b><br>(<i>minutes</i>)</td>"
    html "<td valign=\"top\"><b>Timeout</b><br>(<i>minutes</i>)</td></tr>"

    foreach vtime $page_list {
        set subrec(from) [lindex $record($vtime) 0]
        if {$subrec(from) == "-"} {
            set subrec(from) "!!_Manual_or_Initial_Load_!!"
        }
        set subrec(to) [lindex $record($vtime) 1]
        set subrec(timeout) [lindex $record($vtime) 2]
        if {$page_index > 0} {
            if {[string last $last(to) $subrec(from)] < 0} {
                html "<tr><td><font color=\"red\"><b>$subrec(from)</b></font></td>"
            } else {
                html "<tr><td>$subrec(from)</td>"
            }
            if {$last(to) == $subrec(to)} {
                html "<td><font color=\"#00ff00\"><b>$subrec(to)</b></font></td>"
            } else {
                html "<td>$subrec(to)</td>"
            }
        } else {
            if {$subrec(from) !=  "!!_Manual_or_Initial_Load_!!"} {
                html "<tr><td><a href=\"$subrec(from)\">$subrec(from)</a></td>"
            } else {
                html "<tr><td>$subrec(from)</td>"
            }
            html "<td>$subrec(to)</td>"
        }
        html "<td>[clock format $vtime]</td>"
        if {[incr page_index] < $page_list_len} {
            set visit_time [expr [expr [lindex $page_list $page_index] - $vtime] / 60.0]
            html "<td>$visit_time</td>"
        } else {
            html "<td></td>"
        }
        html "<td>$subrec(timeout)</td></tr>"
        set last(to) $subrec(to)
        set last(from) $subrec(from)
    }
    html "</table><p>"
}
